    #import "NativePDFRendererPlugin.h"
    //#import <native_pdf_renderer2/native_pdf_renderer2-Swift.h>

    @implementation NativePDFRendererPlugin
    
    FlutterMethodChannel* __channel;


    + (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
        FlutterMethodChannel* channel = [FlutterMethodChannel
          methodChannelWithName:@"native_pdf_renderer2"
                binaryMessenger:[registrar messenger]];
          NativePDFRendererPlugin* instance = [[NativePDFRendererPlugin alloc] init];
          [registrar addMethodCallDelegate:instance channel:channel];
          __channel = channel;
    }


- (instancetype)init
{
    self = [super init];
    if (self) {
        self.pages = [[PageRepository alloc] init];
        self.documents = [[DocumentRepository alloc] init];
    }
    return self;
}


    - (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
        if ([@"open.document.data" isEqualToString:call.method]) { [self openDocumentDataHandlerCall:call result:result]; }
        else if ([@"open.document.file" isEqualToString:call.method]) {[self openDocumentFileHandlerCall:call result:result];}
        else if ([@"open.document.asset" isEqualToString:call.method]) {[self openDocumentAssetHandlerCall:call result:result];}
        else if ([@"open.page" isEqualToString:call.method]) {[self openPageHandlerCall:call result:result];}
        else if ([@"close.document" isEqualToString:call.method]) {[self closeDocumentHandlerCall:call result:result];}
        else if ([@"close.page" isEqualToString:call.method]) {[self closePageHandlerCall:call result:result];}
        else if ([@"render" isEqualToString:call.method]) {[self renderHandlerCall:call result:result];}
        else{
            result(FlutterMethodNotImplemented);
        }
    }



-(void) openDocumentDataHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    if ([call.arguments isKindOfClass: [FlutterStandardTypedData class]]){
        FlutterStandardTypedData * data = (FlutterStandardTypedData *) call.arguments;
        
        CGPDFDocumentRef renderer = [self openDataDocument:data.data];
        result( [self.documents registerRenderer:renderer].infoMap );
        
    }else{
        result([FlutterError errorWithCode:@"RENDER_ERROR" message:@"Arguments not sended" details:nil]);
    }
    
}

-(CGPDFDocumentRef) openDataDocument: (NSData *) data {
    CGDataProviderRef cdp =  CGDataProviderCreateWithCFData((CFDataRef)data);
    return CGPDFDocumentCreateWithProvider(cdp);
}
-(CGPDFDocumentRef) openFileDocument: (NSString *) path {
  
    return CGPDFDocumentCreateWithURL(CFBridgingRetain( [NSURL URLWithString:path]));
}
-(CGPDFDocumentRef) openAssetDocument: (NSString *) name {
  
    NSString * path = [NSBundle.mainBundle pathForResource:[NSString stringWithFormat:@"Frameworks/App.framework/flutter_assets/%@", name] ofType:@""];
    
    
    return CGPDFDocumentCreateWithURL(CFBridgingRetain( [NSURL URLWithString:path]));
}



-(void) openDocumentFileHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    if ([call.arguments isKindOfClass: [NSString class]]){
           NSString * data = (NSString *) call.arguments;
           
           CGPDFDocumentRef renderer = [self openFileDocument:data];
           result( [self.documents registerRenderer:renderer].infoMap );
           
       }else{
           result([FlutterError errorWithCode:@"RENDER_ERROR" message:@"Arguments not sended" details:nil]);
       }
    
}
-(void) openDocumentAssetHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    
    if ([call.arguments isKindOfClass: [NSString class]]){
              NSString * name = (NSString *) call.arguments;
              
              CGPDFDocumentRef renderer = [self openAssetDocument:name];
              result( [self.documents registerRenderer:renderer].infoMap );
              
          }else{
              result([FlutterError errorWithCode:@"RENDER_ERROR" message:@"Arguments not sended" details:nil]);
          }
       
}
-(void) openPageHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    if ([call.arguments isKindOfClass: [NSDictionary class]]){
            NSDictionary * args = (NSDictionary*) call.arguments;
            NSString* documentId = (NSString*)[args objectForKey:@"documentId"];
            size_t * pageNumber = (size_t)[args objectForKey:@"documentId"];
            Document * document = (Document * )[self.documents get:documentId];
            CGPDFPageRef renderer = [document openPage:pageNumber];
            Page * p = [self.pages registerDocumentID:documentId Renderer:renderer];
            result(p.infoMap);
            }else{
                result([FlutterError errorWithCode:@"RENDER_ERROR" message:@"Arguments not sended" details:nil]);
            }
}
-(void) closePageHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    if ([call.arguments isKindOfClass: [NSString class]]){
        NSString * id_ = (NSString *) call.arguments;
          [self.pages close:id_];
       
        
    }
    result( nil );
}
-(void) renderHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    
    if ([call.arguments isKindOfClass: [NSDictionary class]]){
              NSDictionary * args = (NSDictionary*) call.arguments;
              NSString* pageId = (NSString*)[args objectForKey:@"pageId"];
              size_t width = (size_t)[args valueForKey:@"width"];
              size_t height = (size_t)[args valueForKey:@"height"];
              Boolean crop = (Boolean)[args valueForKey:@"crop"];
              int compressFormat = (int)[args valueForKey:@"format"];
              NSString* backgroundColor = (NSString*)[args objectForKey:@"backgroundColor"];
        
                CGRect * cropZone = nil;
            //implementing crop zone;
        
            dispatch_async(dispatch_get_global_queue( DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                NSDictionary * results = nil;
                Page * p =  [self.pages get:pageId];
                if (p != nil){
                    DataResult* d =  [p renderWidth:width height:height crop:cropZone compressFormat: ((compressFormat == 0) ? JPEG : PNG) backgroundColor: [UIColor whiteColor]];
                    //hexCOlor implementing
                    
                    if (d!=nil){
                        results = @{
                            @"width":[NSNumber numberWithInt: (int)width],
                            @"height":[NSNumber numberWithInt: (int)height],
                            @"data":[FlutterStandardTypedData typedDataWithBytes:d.data]
                        };
                        result(results);
                    }
                }
                
            });
        
        
              }else{
                  result([FlutterError errorWithCode:@"RENDER_ERROR" message:@"Arguments not sended" details:nil]);
              }
}
-(void) closeDocumentHandlerCall: (FlutterMethodCall*)call result:(FlutterResult)result{
    if ([call.arguments isKindOfClass: [NSString class]]){
              NSString * id_ = (NSString *) call.arguments;
                [self.documents close:id_];
             
              
          }
     result( nil );
}

    @end
