//
//  Page.m
//  Pods
//
//  Created by Blazej Zyglarski on 11/03/2020.
//

#import "Page.h"

@implementation Page
@synthesize id_;
@synthesize renderer;
@synthesize documentId;
@synthesize boxRect;

- (id) initWithId: (NSString *) id_ documentId:(NSString*) documentID renderer:(CGPDFPageRef) renderer {
    self = [super init];
    self.id_ = id_;
    self.documentId = documentId;
    self.renderer = renderer;
    
    self.boxRect = CGPDFPageGetBoxRect(renderer, kCGPDFMediaBox);
    return self;
    
}

- (size_t) getPageNumber{
    return CGPDFPageGetPageNumber(renderer);
}

- (size_t) getWidth {
    return self.boxRect.size.width;
}


- (size_t) getHeight {
    return self.boxRect.size.height;
}

- (NSDictionary*) getInfoMap {
    return @{
        @"id": self.id_,
         @"documentId": self.documentId,
        @"pageNumber": [NSNumber numberWithUnsignedLong:self.pageNumber],
        @"width": [NSNumber numberWithUnsignedLong:self.width],
        @"height": [NSNumber numberWithUnsignedLong:self.height]
        
    };
}

-(DataResult*) renderWidth: (size_t) width height:(size_t) height crop: (CGRect*) crop compressFormat:(CompressFormat) compressFormat backgroundColor: (UIColor*) backgroundColor{
    
    CGRect pdfBBox = CGPDFPageGetBoxRect(renderer, kCGPDFMediaBox);
    size_t stride = width * 4;
    NSMutableData * tempData = [[NSMutableData alloc] initWithBytes:nil length: stride * height ];
    NSData * data;
    Boolean success;
    size_t sx = (CGFloat)width / (CGFloat)pdfBBox.size.width;
    size_t sy = (CGFloat)height / (CGFloat)pdfBBox.size.height;
    //tempData
    NSDictionary *auxillaryInformation = @{(NSString *)kCGPDFContextCreator: @"asuri.pl"};

    UIGraphicsBeginPDFContextToData(tempData, pdfBBox, auxillaryInformation);
    
    CGContextRef PDFContext = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(PDFContext, backgroundColor.CGColor);
    CGContextFillRect(PDFContext, pdfBBox);
    CGContextDrawPDFPage(PDFContext, renderer);
    
    CGImageRef  cgimageref = CGBitmapContextCreateImage(PDFContext);
    UIImage* img = [UIImage imageWithCGImage:cgimageref];
    
    switch (compressFormat) {
        case JPEG:
            data = UIImageJPEGRepresentation(img, 1.0);
            break;
        case PNG:
            data = UIImagePNGRepresentation(img);
            break;
    }
    
    return [[DataResult alloc]initWithWidth:0 height:0 data:data];
    
    
    
    
}





@end
